@@ -17,7 +17,6 @@ module Data.Yaml.Internal
 import Control.Applicative ((<$>), Applicative(..))
 #endif
 import Control.Exception
-import Control.Exception.Enclosed
 import Control.Monad (liftM, ap, unless)
 import Control.Monad.IO.Class (MonadIO, liftIO)
 import Control.Monad.Trans.Class (MonadTrans, lift)
@@ -133,12 +132,24 @@ requireEvent e = do
 
 parse :: C.Sink Event Parse Value
 parse = do
-    requireEvent EventStreamStart
-    requireEvent EventDocumentStart
-    res <- parseO
-    requireEvent EventDocumentEnd
-    requireEvent EventStreamEnd
-    return res
+    streamStart <- CL.head
+    case streamStart of
+        Nothing ->
+            -- empty string input
+            return Null
+        Just EventStreamStart -> do
+            documentStart <- CL.head
+            case documentStart of
+                Just EventStreamEnd ->
+                    -- empty file input, comment only string/file input
+                    return Null
+                Just EventDocumentStart -> do
+                    res <- parseO
+                    requireEvent EventDocumentEnd
+                    requireEvent EventStreamEnd
+                    return res
+                _ -> liftIO $ throwIO $ UnexpectedEvent documentStart Nothing
+        _ -> liftIO $ throwIO $ UnexpectedEvent streamStart Nothing
 
 parseScalar :: ByteString -> Anchor -> Style -> Tag
             -> C.Sink Event Parse Text
@@ -241,7 +252,10 @@ decodeHelper :: FromJSON a
              => C.Source Parse Y.Event
              -> IO (Either ParseException (Either String a))
 decodeHelper src = do
-    x <- tryAny $ runResourceT $ flip evalStateT Map.empty $ src C.$$ parse
+    -- This used to be tryAny, but the fact is that catching async
+    -- exceptions is fine here. We'll rethrow them immediately in the
+    -- otherwise clause.
+    x <- try $ runResourceT $ flip evalStateT Map.empty $ src C.$$ parse
     case x of
         Left e
             | Just pe <- fromException e -> return $ Left pe
@@ -253,7 +267,7 @@ decodeHelper_ :: FromJSON a
               => C.Source Parse Event
               -> IO (Either ParseException a)
 decodeHelper_ src = do
-    x <- tryAny $ runResourceT $ flip evalStateT Map.empty $ src C.$$ parse
+    x <- try $ runResourceT $ flip evalStateT Map.empty $ src C.$$ parse
     return $ case x of
         Left e
             | Just pe <- fromException e -> Left pe
@@ -270,10 +284,11 @@ specialStrings = HashSet.fromList $ T.words
     "y Y yes Yes YES n N no No NO true True TRUE false False FALSE on On ON off Off OFF null Null NULL ~"
 
 isNumeric :: Text -> Bool
-isNumeric =
-    T.all isNumeric'
+isNumeric t =
+    T.all isNumeric' t && T.any isNumber t
   where
-    isNumeric' c = ('0' <= c && c <= '9')
+    isNumber c = '0' <= c && c <= '9'
+    isNumeric' c = isNumber c
                 || c == 'e'
                 || c == 'E'
                 || c == '.'
