@@ -70,6 +70,7 @@ spec = do
         it "encode/decode strings" caseEncodeDecodeStrings
         it "decode invalid file" caseDecodeInvalid
         it "processes datatypes" caseDataTypes
+        it "encode invalid numbers" caseEncodeInvalidNumbers
     describe "Data.Yaml.Pretty" $ do
         it "encode/decode" caseEncodeDecodeDataPretty
         it "encode/decode strings" caseEncodeDecodeStringsPretty
@@ -95,7 +96,11 @@ spec = do
         it "parses when all uppercase" caseUppercaseBool
         it "parses when titlecase" caseTitlecaseBool
     describe "empty input" $ do
-        it "doesn't crash" caseEmptyInput
+        it "parses as Null" caseEmptyInput
+        it "parses as Null from file" caseEmptyInputFile
+    describe "comment only input" $ do
+        it "parses as Null" caseCommentOnlyInput
+        it "parses as Null from file" caseCommentOnlyInputFile
     describe "alias stripping" $ do
         it "works" caseStripAlias
     describe "nulls" $ do
@@ -430,6 +435,10 @@ caseDataTypes =
         , ("null", D.Null)
         ]
 
+caseEncodeInvalidNumbers :: Assertion
+caseEncodeInvalidNumbers =
+    D.encode (D.String ".") @?= ".\n"
+
 caseDataTypesPretty :: Assertion
 caseDataTypesPretty =
     D.decode (Pretty.encodePretty Pretty.defConfig val) @?= Just val
@@ -446,7 +455,7 @@ caseQuotedNumber, caseUnquotedNumber, caseAttribNumber, caseIntegerDecimals :: A
 caseQuotedNumber = D.decode "foo: \"1234\"" @?= Just (object [("foo", D.String "1234")])
 caseUnquotedNumber = D.decode "foo: 1234" @?= Just (object [("foo", D.Number 1234)])
 caseAttribNumber = D.decode "foo: !!str 1234" @?= Just (object [("foo", D.String "1234")])
-caseIntegerDecimals = D.encode (1 :: Int) @?= "1\n...\n"
+caseIntegerDecimals = D.encode (1 :: Int) @?= "1\n"
 
 obj :: Maybe D.Value
 obj = Just (object [("foo", D.Bool False), ("bar", D.Bool True), ("baz", D.Bool True)])
@@ -457,7 +466,20 @@ caseUppercaseBool = D.decode "foo: FALSE\nbar: Y\nbaz: ON" @?= obj
 caseTitlecaseBool = D.decode "foo: No\nbar: Yes\nbaz: True" @?= obj
 
 caseEmptyInput :: Assertion
-caseEmptyInput = D.decode B8.empty @?= (Nothing :: Maybe D.Value)
+caseEmptyInput = D.decodeEither B8.empty @?= Right D.Null
+
+caseEmptyInputFile :: Assertion
+caseEmptyInputFile = do
+    out <- D.decodeFileEither "test/resources/empty.yaml"
+    either (Left . D.prettyPrintParseException) Right out @?= Right D.Null
+
+caseCommentOnlyInput :: Assertion
+caseCommentOnlyInput = D.decodeEither "# comment\n" @?= Right D.Null
+
+caseCommentOnlyInputFile :: Assertion
+caseCommentOnlyInputFile = do
+    out <- D.decodeFileEither "test/resources/empty2.yaml"
+    either (Left . D.prettyPrintParseException) Right out @?= Right D.Null
 
 checkNull :: T.Text -> Spec
 checkNull x =
